@@ -3,10 +3,10 @@
 # The code implements some seemingly awkward constructs and redundant functionality
 # This is however required for backwards compatibility, we'll get rid of it eventually
 #
-from .safrs_api import DB, log, SAFRSAPI, SAFRS, dict_merge, test_decorator, SAFRSRequest
+from .safrs_init import DB, log, SAFRS, dict_merge, test_decorator, SAFRSRequest
 from .errors import ValidationError, GenericError, UnAuthorizedError, NotFoundError
 from .json_encoder import SAFRSJSONEncoder
-from ._api import Api, SAFRSRestAPI
+from .safrs_api import SAFRSRestAPI, SAFRSAPI
 from .base import SAFRSBase
 from .jsonapi_attr import jsonapi_attr
 from .jsonapi import jsonapi_format_response, SAFRSFormattedResponse, paginate
@@ -14,6 +14,7 @@ from .api_methods import search, startswith
 from .swagger_doc import jsonapi_rpc
 from .__about__ import __version__, __description__
 
+
 __all__ = (
     "__version__",
     "__description__",
