@@ -283,7 +283,8 @@ def get_swagger_doc_arguments(cls, method_name, http_method):
             # model = SchemaClassFactory(model_name, [])
             # arg_field = {"schema": model, "type": "string"} # tbd?
             method_field = {"method": method_name, "args": args}
-            fields["meta"] = schema_from_object(model_name, method_field)
+            if getattr(method, "valid_jsonapi", True):
+                fields["meta"] = schema_from_object(model_name, method_field)
 
         for param in parameters:
             if param.get("in") is None:
@@ -379,8 +380,12 @@ def swagger_doc(cls, tags=None):
 
             # Create the default POST body schema
             sample_dict = cls._s_sample_dict()
-            # The POST sample doesn't contain an "id"
+            # The POST sample doesn't contain an "id", unless cls.allow_client_generated_ids is True
             sample_data = schema_from_object(inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type}})
+            if cls.allow_client_generated_ids:
+                sample_data = schema_from_object(
+                    inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type, "id": "client_generated"}}
+                )
             parameters.append(
                 {
                     "name": "POST body",
